package com.fireworksdk.bridge.flutter.handler

import android.app.Activity
import com.fasterxml.jackson.core.type.TypeReference
import com.fireworksdk.bridge.flutter.models.FWShoppingModuleCallMethod
import com.fireworksdk.bridge.flutter.pages.FWVideoShoppingCartActivity
import com.fireworksdk.bridge.flutter.utils.FWEventUtils
import com.fireworksdk.bridge.models.FWVideoShoppingProduct
import com.fireworksdk.bridge.models.weakProperty
import com.fireworksdk.bridge.utils.FWJsonUtils
import com.fireworksdk.bridge.utils.FWLogUtils
import com.loopnow.fireworklibrary.baya.Baya
import com.loopnow.fireworklibrary.baya.UpdateCartStatus
import com.loopnow.fireworklibrary.data.Product
import com.loopnow.fireworklibrary.utils.Util
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.Result


class FWShoppingModuleHandler(
    currentActivity: Activity?,
    channel: MethodChannel?
): MethodChannel.MethodCallHandler {

    private val methodChannel by weakProperty<MethodChannel?>(channel)
    private val currentActivity by weakProperty<Activity?>(currentActivity)

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            FWShoppingModuleCallMethod.Init.rawValue -> {
                cartListener()
                productListener()
                result.success(null)
            }
            FWShoppingModuleCallMethod.SetCartIconVisible.rawValue -> {
                val value: Boolean? = call.arguments as Boolean?
                Baya.willUseCustomCart(value?:true)
                result.success(null)
            }
            FWShoppingModuleCallMethod.SetCartItemCount.rawValue -> {
                val value: Int? = call.arguments as Int?
                Baya.itemCount.value = value?:0
                result.success(null)
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    private fun cartListener() {
        Baya.cartInterface = object: Baya.CartInterface {

            override fun updateCart(
                activity: Activity,
                productId: String,
                unitId: String,
                quantity: Int
            ) {
                FWLogUtils.d { "FWVideoShoppingModule updateCart, productId: $productId, unitId: $unitId" }
                FWEventUtils.sendAddToCartClickEvent(methodChannel, productId, unitId, object: Result {
                    override fun success(result: Any?) {
                        FWLogUtils.d { "FWVideoShoppingModule updateAddToCartStatus: $result" }
                        if (result is Map<*, *>) {
                            val map: HashMap<String, String> = result as HashMap<String, String>
                            map["res"].let { res ->
                                val tip = map["tips"]
                                when (res) {
                                    "success" -> {
                                        Baya.updateCartStatus(UpdateCartStatus.Success(tip, unitId))
                                    }
                                    "loading" -> {
                                        Baya.updateCartStatus(UpdateCartStatus.Loading)
                                    }
                                    else -> {
                                        Baya.updateCartStatus(UpdateCartStatus.Error(tip, unitId))
                                    }
                                }
                            }
                        }
                    }

                    override fun error(
                        errorCode: String?,
                        errorMessage: String?,
                        errorDetails: Any?
                    ) {
                        Baya.updateCartStatus(UpdateCartStatus.Error(errorMessage, unitId))
                    }

                    override fun notImplemented() {
                    }

                })
            }

            override fun cartClicked(activity: Activity) {
                FWLogUtils.d { "FWVideoShoppingModule cartClicked" }
                activity.startActivity(FWVideoShoppingCartActivity.createIntent(activity))
            }
        }
    }

    private fun productListener() {
        Baya.productInterface = object: Baya.ProductInterface {
            override fun hydrateProducts(videoId: String, products: List<Product>) {
                FWLogUtils.d { "FWVideoShoppingModule hydrateProducts, videoId: $videoId" }
                if (videoId.isNullOrBlank() || products.isNullOrEmpty()) {
                    return
                }
                FWEventUtils.sendUpdateProductsDetailsEvent(methodChannel, products, object: Result {
                    override fun success(result: Any?) {
                        FWLogUtils.d { "FWVideoShoppingModule updateVideoProduct: $result" }
                        if (result is ArrayList<*>) {
                            updateVideoProducts(result, videoId, products)
                        }
                    }

                    override fun error(
                        errorCode: String?,
                        errorMessage: String?,
                        errorDetails: Any?
                    ) {
                    }

                    override fun notImplemented() {
                    }
                })
            }

            override fun displayProductInfo(
                productId: String,
                unitId: String,
                productWebUrl: String?
            ): Boolean {
                return false
            }
        }
    }

    private fun updateVideoProducts(productArray: ArrayList<*>?, videoId: String, products: List<Product>) {

        val videoShoppingProducts = FWJsonUtils.fromJson(FWJsonUtils.toJson(productArray), object : TypeReference<List<FWVideoShoppingProduct>?>() {})
        if (videoShoppingProducts.isNullOrEmpty()) {
            return
        }

        for (product in products) {
            for (videoShoppingProduct in videoShoppingProducts) {
                if (product.extId == videoShoppingProduct.productId) {

                    if (videoShoppingProduct.name != null) {
                        product.name = videoShoppingProduct.name
                    }

                    if (videoShoppingProduct.description != null) {
                        product.description = videoShoppingProduct.description
                    }

                    if (videoShoppingProduct.units != null) {
                        for (videoShoppingProductUnit in videoShoppingProduct.units) {
                            for (productUnit in product.productUnits) {
                                if (videoShoppingProductUnit.unitId.equals(productUnit.extId)) {
                                    productUnit.name = videoShoppingProductUnit.name
                                    productUnit.price = videoShoppingProductUnit.price?.amount
                                    productUnit.currencyCode = videoShoppingProductUnit.price?.currencyCode
                                    productUnit.priceLiteral = Util.getPriceLocaleFormatter(productUnit.price, productUnit.currencyCode)
                                }
                            }
                        }
                    }

                }
            }
        }
        Baya.updateProductsComplete(videoId)
    }
}