//
//  FireworkSDKModulePlugin.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/2/21.
//

import Foundation
import Flutter
import FireworkVideo

//var gShareBaseURL: String?
//var gAdBadgeConfiguration: AdBadgeConfiguration?


class FireworkSDKModulePlugin: NSObject, FlutterPlugin, FireworkVideoSDKDelegate {

    var customCTAClickEnabled: Bool = false
    
    /// Decide whether to send video playback event to RN.
    var enableVideoPlayBackEvent: Bool = false
    
    var channel: FlutterMethodChannel! = nil
    
    struct ShareBaseURLConfiguration {
        static var shareBaseURL: String?
    }
    
    struct FWAdBadgeConfiguration {
        static var adBadgeConfiguration: AdBadgeConfiguration?

        static var fwAdBadgeConfiguration: FireworkVideo.AdBadgeConfiguration? {
            guard let adBadgeConfiguration = self.adBadgeConfiguration else {
                return nil
            }

            var configuration = FireworkVideo.AdBadgeConfiguration()
            if let textColor = adBadgeConfiguration.textColor {
                configuration.textColor = textColor.uicolor()
            }
            if let backgroundColor = adBadgeConfiguration.backgroundColor {
                configuration.backgroundColor = backgroundColor.uicolor()
            }
            switch adBadgeConfiguration.badgeTextType {
                case .sponsored:
                    configuration.badgeText = FireworkVideo.AdBadgeConfiguration.BadgeText.sponsored
                default:
                    configuration.badgeText = FireworkVideo.AdBadgeConfiguration.BadgeText.ad
            }

            return configuration
        }
    }
    
    
    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: FWChannelName.FireworkSdkModule.rawValue, binaryMessenger: registrar.messenger())
        let instance = FireworkSDKModulePlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case FWSDKModuleMethodName.`init`.rawValue:
            initializeSDK(call.arguments as? [String: AnyObject])
            result(nil)
        case FWSDKModuleMethodName.openVideoPlayer.rawValue:
            openVideoPlayer(call.arguments as? [String: AnyObject])
            result(nil)
        case FWSDKModuleMethodName.setCustomCTAClickEnabled.rawValue:
            setCustomCTAClickEnabled(call.arguments as? Bool)
            result(nil)
        case FWSDKModuleMethodName.setVideoPlaybackEventEnabled.rawValue:
            setVideoPlaybackEventEnabled(call.arguments as? Bool)
            result(nil)
        case FWSDKModuleMethodName.setShareBaseURL.rawValue:
            setShareBaseURL(call.arguments as? String)
            result(nil)
        case FWSDKModuleMethodName.setAdBadgeConfiguration.rawValue:
            setAdBadgeConfiguration(call.arguments as? [String: AnyObject])
            result(nil)
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    private func initializeSDK(_ config: [String: AnyObject]?) {
        DispatchQueue.once {
            guard let config = config else {
                return
            }
            
            FireworkVideoSDK.initializeSDK(delegate: self, userID: config["userId"] as? String)
            FireworkVideoSDK.ctaDelegate = self
            FireworkVideoSDK.eventTracking.videoPlaybackDelegate = self
            FireworkVideoSDK.eventTracking.feedDelegate = self
        }
    }
    
    private func openVideoPlayer(_ config: [String: AnyObject]?) {
        guard let urlString = config?["url"] as? String, let url = URL.init(string: urlString) else {
            return
        }
        
        var finalConfig = RCTConvert.videoPlayerContentConfiguration(config?["config"] as? [String: AnyObject])
        if let baseUrl = FireworkSDKModulePlugin.ShareBaseURLConfiguration.shareBaseURL {
            finalConfig.shareButton.behavior.baseURL = URL(string: baseUrl)
        }
        
        guard let presentedVC = UIViewController.presentedViewController() else {
            return
        }

        var feedContentConfig = VideoFeedContentConfiguration()
        feedContentConfig.playerView = finalConfig
        if let adBadge = FireworkSDKModulePlugin.FWAdBadgeConfiguration.fwAdBadgeConfiguration {
            feedContentConfig.adBadge = adBadge
        }

        VideoFeedViewController.openVideoPlayer(with: url, feedContentConfig, presentedVC) { _ in

        }
    }
    
    private func setCustomCTAClickEnabled(_ enabled: Bool?) {
        if let enabled = enabled {
            customCTAClickEnabled = enabled
        }
    }
    
    private func setVideoPlaybackEventEnabled(_ enabled: Bool?) {
        if let enabled = enabled {
            enableVideoPlayBackEvent = enabled
        }
    }
    
    func setShareBaseURL(_ url: String?) {
        if let url = url {
            FireworkSDKModulePlugin.ShareBaseURLConfiguration.shareBaseURL = url
        }
    }
    
    func setAdBadgeConfiguration(_ config: [String: AnyObject]?) {
//        FireworkSDKModulePlugin.FWAdBadgeConfiguration.adBadgeConfiguration = RCTConvert.adBadgeConfiguration(config)
        FireworkSDKModulePlugin.FWAdBadgeConfiguration.adBadgeConfiguration = JsonHelper.fromDictionary(config, type: AdBadgeConfiguration.self)
    }
    
    // MARK: - FireworkVideoSDKDelegate
    func fireworkVideoDidLoadSuccessfully() {
        channel.invokeMethod(FWEventName.SDKInit.rawValue, arguments: [:])
    }
    
    func fireworkVideoDidLoadWith(error: FireworkVideoSDKError) {
        channel.invokeMethod(FWEventName.SDKInit.rawValue, arguments: ["error": ["name": error.jsErrorName, "reason": error.recoverySuggestion ?? "default"]])
    }
}
