import '../../models/fw_error.dart';
import '../../models/video_feed_configuration.dart';
import '../../models/video_feed_mode.dart';
import '../../models/video_feed_source.dart';
import '../../models/video_player_configuration.dart';
import 'video_feed_platform_handler.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';

import '../../utils/model_utils/video_feed_configuration_util.dart';
import '../../utils/model_utils/video_feed_mode_util.dart';
import '../../utils/model_utils/video_feed_source_util.dart';
import '../../utils/model_utils/video_player_configuration_util.dart';

typedef VideoFeedPlatformViewLoadFinishedCallback = void Function(
    FWError? error);
typedef VideoFeedPlatformViewCreatedCallback = void Function(
    VideoFeedPlatformHandler handler);

class VideoFeedPlatformView extends StatefulWidget {
  final VideoFeedSource source;
  final String? channel;
  final String? playlist;
  final String? playlistGroup;
  final Map<String, List<String>>? dynamicContentParameters;
  final VideoFeedMode mode;
  final VideoFeedConfiguration? videoFeedConfiguration;
  final VideoPlayerConfiguration? videoPlayerConfiguration;
  final bool useHybridCompositionForAndroid;
  final VideoFeedPlatformViewLoadFinishedCallback?
      onVideoFeedPlatformViewLoadFinished;
  final VideoFeedPlatformViewCreatedCallback? onVideoFeedPlatformViewCreated;

  const VideoFeedPlatformView({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.playlistGroup,
    this.dynamicContentParameters,
    this.mode = VideoFeedMode.row,
    this.videoFeedConfiguration,
    this.videoPlayerConfiguration,
    this.useHybridCompositionForAndroid = false,
    this.onVideoFeedPlatformViewLoadFinished,
    this.onVideoFeedPlatformViewCreated,
  }) : super(key: key);

  @override
  _VideoFeedPlatformViewState createState() => _VideoFeedPlatformViewState();
}

class _VideoFeedPlatformViewState extends State<VideoFeedPlatformView> {
  final String _viewType = "fw.fondor.view/video_feed";
  VideoFeedPlatformHandler? _handler;

  @override
  void initState() {
    super.initState();
  }

  @override
  void didUpdateWidget(covariant VideoFeedPlatformView oldWidget) {
    super.didUpdateWidget(oldWidget);
    _handler?.updateWidget(widget);
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.android) {
      return _buildAndroidPlatformView(context);
    } else {
      return _buildIOSPlatformView(context);
    }
  }

  Widget _buildAndroidPlatformView(BuildContext context) {
    final creationParams = _generateCreationParams();
    final layoutDirection =
        Directionality.maybeOf(context) ?? TextDirection.ltr;
    const creationParamsCodec = StandardMessageCodec();

    if (widget.useHybridCompositionForAndroid) {
      return PlatformViewLink(
        viewType: _viewType,
        surfaceFactory:
            (BuildContext context, PlatformViewController controller) {
          return AndroidViewSurface(
            controller: controller as AndroidViewController,
            gestureRecognizers: _generateGestureRecognizers(),
            hitTestBehavior: PlatformViewHitTestBehavior.opaque,
          );
        },
        onCreatePlatformView: (PlatformViewCreationParams params) {
          return PlatformViewsService.initSurfaceAndroidView(
            id: params.id,
            viewType: _viewType,
            layoutDirection: layoutDirection,
            creationParams: creationParams,
            creationParamsCodec: creationParamsCodec,
            onFocus: () {
              params.onFocusChanged(true);
            },
          )
            ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
            ..addOnPlatformViewCreatedListener(_onPlatformViewCreated)
            ..create();
        },
      );
    } else {
      return AndroidView(
        viewType: _viewType,
        onPlatformViewCreated: _onPlatformViewCreated,
        layoutDirection: layoutDirection,
        creationParams: creationParams,
        creationParamsCodec: creationParamsCodec,
        gestureRecognizers: _generateGestureRecognizers(),
      );
    }
  }

  Widget _buildIOSPlatformView(BuildContext context) {
    Map<String, dynamic> creationParams = _generateCreationParams();
    final layoutDirection =
        Directionality.maybeOf(context) ?? TextDirection.ltr;

    return UiKitView(
      viewType: _viewType,
      onPlatformViewCreated: _onPlatformViewCreated,
      layoutDirection: layoutDirection,
      creationParams: creationParams,
      creationParamsCodec: const StandardMessageCodec(),
      gestureRecognizers: _generateGestureRecognizers(),
    );
  }

  Map<String, dynamic> _generateCreationParams() {
    Map<String, dynamic> creationParams = <String, dynamic>{
      "source": VideoFeedSourceUtil.convertToRawFeedSource(widget.source),
    };

    if (widget.channel != null) {
      creationParams["channel"] = widget.channel!;
    }

    if (widget.playlist != null) {
      creationParams["playlist"] = widget.playlist!;
    }

    if (widget.playlistGroup != null) {
      creationParams["playlistGroup"] = widget.playlistGroup!;
    }

    if (widget.dynamicContentParameters != null) {
      creationParams["dynamicContentParameters"] =
          widget.dynamicContentParameters!;
    }

    creationParams["mode"] =
        VideoFeedModeUtil.convertToRawFeedMode(widget.mode);

    if (widget.videoFeedConfiguration != null) {
      creationParams["videoFeedConfiguration"] =
          VideoFeedConfigurationUtil.toJson(widget.videoFeedConfiguration!);
    }

    if (widget.videoPlayerConfiguration != null) {
      creationParams["videoPlayerConfiguration"] =
          VideoPlayerConfigurationUtil.toJson(widget.videoPlayerConfiguration!);
    }
    return creationParams;
  }

  Set<Factory<OneSequenceGestureRecognizer>> _generateGestureRecognizers() {
    return <Factory<OneSequenceGestureRecognizer>>{
      Factory<OneSequenceGestureRecognizer>(
        () => widget.mode == VideoFeedMode.row
            ? HorizontalDragGestureRecognizer()
            : VerticalDragGestureRecognizer(),
      ),
    };
  }

  void _onPlatformViewCreated(int viewId) {
    _handler = VideoFeedPlatformHandler(viewId: viewId, widget: widget);
    widget.onVideoFeedPlatformViewCreated?.call(_handler!);
  }
}
